/******************************************************************************/
/*                                                                            */
/* File-Name       : astropalm.c                                              */
/* Project         : astropalm                                                */
/* Compiler        : PRC-Tools ( gcc )                                        */
/* OperatingSystem : Palm OS  (5.xx)                                          */
/* Author          : J. Kuenstner                                             */
/* Date            : 2005-03-23                                               */
/* Description     : Main-Module for astropalm                                */
/*                                                                            */
/******************************************************************************/
/*                               HISTORY                                      */
/*.CHANGE_CONTROL

 vers   | date     |  by         | reason
--------+----------+-------------+------------------------------------------
  1.0   |2003-10-??|  JKue       | File created
--------+----------+-------------+------------------------------------------
  1.6   |2003-10-24|  JKue       | - Bugfix DateTimeToIndex(), ignored yearbreak during capricorn
                                   - support for scroll-keys
								   - added the Attributes
--------+----------+-------------+------------------------------------------
  1.7   |2003-10-27|  JKue       | - Bugfixes in the texts ( thanks to Ralf Brinkmann )
                                   - Added support for 5way-Navi ( ~ 5h-10h to find the necessary info ... )
--------+----------+-------------+------------------------------------------
  1.8   |2003-11-??|  JKue       | - Added the symbols 
--------+----------+-------------+------------------------------------------
  1.9   |2003-11-21|  JKue       | - Bugfix, output-area covered buttons => unpredictable ... 
--------+----------+-------------+------------------------------------------
  2.0   |2005-03-23|  JKue       | - Bugfix in 5way-Key-Handling, longpress now enters launcher as expected

*/


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include <PalmOS.h>
#include "PalmNavigator.h"
#include "astropalm.h"

/**
 * Description of a zodiac.
 * Strings in german and english, the start and end-date, a bitmap with the stars, a bitmap with the icon and the attributes
 */
typedef struct tag_MyZodiac
{
	char * szGerman;			/** <- Name of the zodiac in german */
	char * szEnglish;			/** <- Name of the zodiac in english */
	DateType Start;				/** <- Starting date */
	DateType End;				/** <- End date */
	int      BitmapId;			/** <- the name-giving star-constellation */
	int      IcoId;				/** <- the icon as it is known in astrology */ 
	char * pszAttributes;		/** <- the attributes of the zodiac */ 
}t_Zodiac;


t_Zodiac ZodiacSigns[] =
{
	{"Wassermann","Aquarius",{ 00,1,20},{00,2,18}, PictureAquarius, IcoAquarius, "selbstndig, menschlich, originell, aufrichtig, erfinderisch, distanziert" },
	{"Fische","Pisces",{ 00,2,19},{00,3,20}, PicturePisces,  IcoPisces, "sensibel, fantasievoll, geistesabwesend, liebenswrdig, beeinflussbar"  },
	{"Widder","Aries",{ 00,3,21},{00,4,19}, PictureAries, IcoAries, "dominant, ungeduldig, kraftvoll, aufrichtig, egoistisch, begeisterungsfhig" },
	{"Stier","Taurus",{ 00,4,20},{00,5,20}, PictureTaurus, IcoTaurus, "geduldig, Besitz ergreifend, zuverlssig, wenig Selbstdisziplin, beharrlich, warmherzig" },
	{"Zwilling","Gemini",{ 00,5,21},{00,6,21}, PictureGemini, IcoGemini, "kommunikativ, vielseitig, rastlos, schlagfertig, intellektuell, scharfsinnig" },
	{"Krebs","Cancer",{ 00,6,22},{00,7,22}, PictureCancer, IcoCancer, "sensibel, gefhlsbetont, frsorglich, launisch, fantasievoll, liebevoll" },
	{"Lwe","Leo",{ 00,7,23},{00,8,23}, PictureLeo, IcoLeo, "grozgig, mitteilsam, liebevoll, herrisch, kreativ, imposant" },
	{"Jungfrau","Virgo",{ 00,8,24},{00,9,23}, PictureVirgo, IcoVirgo, "kopfbetont, kritisch, bescheiden, intelligent, pingelig" },
	{"Waage","Libra",{ 00,9,24},{00,10,23}, PictureLibra, IcoLibra, "charmant, gelassen, diplomatisch, kontaktfreudig, leichtglubig, nachtragend" },
	{"Skorpion","Scorpio",{ 00,10,24},{00,11,22}, PictureScorpio, IcoScorpio, "leidenschaftlich, ernsthaft, intuitiv, magnetische Anziehungs- kraft, eiferschtig, getrieben"},
	{"Schtze","Sagittarius",{ 00,11,23},{00,12,21}, PictureSagittarius, IcoSagittarius, "philosophisch, von Forscherdrang beseelt, optimistisch, taktlos" },
	{"Steinbock","Capricorn",{ 00,12,22},{00,1,19}, PictureCapricorn, IcoCapricorn, "umsichtig, berechnend, praktisch veranlagt, ehrgeizig, diszipliniert" }
};


static int ZodiacIndex = 0;			

/**
  Called from the main-program to setup everything 
*/ 

static void StartApplication( void )
{
	DateTimePtr pDateTimeToday;
	pDateTimeToday = MemPtrNew( 20 );
	TimSecondsToDateTime( TimGetSeconds(), pDateTimeToday );
	ZodiacIndex = DateTimeToIndex( pDateTimeToday);
	MemPtrFree( pDateTimeToday );
    FrmGotoForm(ListFormular);
}

/**
  Called from the main-program just before leaving the program.
*/ 

static void StopApplication( void )
{
    FrmCloseAllForms();
}

/**
  Draws the information about the Zodiac in the given formular.
  \param nIndex   -> Index into the zodiac-table
  \param formular -> pointer to the formular
*/

void DrawZodiac( int nIndex, FormPtr formular )
{
	MemHandle hmBitmapMem;
	BitmapType * bitmap;
	char szDateString[30];
	FieldPtr ptrField;

	/* Generate the string for the date */
	StrPrintF( szDateString,"%02d.%02d. - %02d.%02d.",
				ZodiacSigns[nIndex].Start.day,
				ZodiacSigns[nIndex].Start.month,
				ZodiacSigns[nIndex].End.day,
				ZodiacSigns[nIndex].End.month );

	/* Erase the old star-constellation-bitmap */
	hmBitmapMem = DmGetResource( bitmapRsc, PictureErase );
	bitmap = MemHandleLock( hmBitmapMem );
	WinDrawBitmap( bitmap, 10, 45 );
	MemHandleUnlock( hmBitmapMem );

	/* Draw the new star-constellation-bitmap */
	hmBitmapMem = DmGetResource( bitmapRsc, ZodiacSigns[nIndex].BitmapId );
	bitmap = MemHandleLock( hmBitmapMem );
	WinDrawBitmap( bitmap, 10, 45 );
	MemHandleUnlock( hmBitmapMem );

	/* Erase the old zodiac-icon */
	hmBitmapMem = DmGetResource( bitmapRsc, PictureErase );
	bitmap = MemHandleLock( hmBitmapMem );
	WinDrawBitmap( bitmap, 70, 55 );
	MemHandleUnlock( hmBitmapMem );

	/* Draw the new zodiac-icon */
	hmBitmapMem = DmGetResource( bitmapRsc, ZodiacSigns[nIndex].IcoId );
	bitmap = MemHandleLock( hmBitmapMem );
	WinDrawBitmap( bitmap, 70, 55 );
	MemHandleUnlock( hmBitmapMem );

	/* Draw the start and end-date */
	WinDrawChars("                              ",30,40,15 );
	WinDrawChars(szDateString,StrLen(szDateString),40,15 );

	/* draw the name in german  */
	WinDrawChars("                              ",30,10,30 );
	WinDrawChars(ZodiacSigns[nIndex].szGerman,StrLen(ZodiacSigns[nIndex].szGerman),10,30 );

	/* draw the name in english  */
	WinDrawChars("                              ",30,80,30 );
	WinDrawChars(ZodiacSigns[nIndex].szEnglish,StrLen(ZodiacSigns[nIndex].szEnglish),80,30 );

	/* draw the attributes into the textfield */
	ptrField = FrmGetObjectPtr( formular, FrmGetObjectIndex( formular, Feld ));
	FldSetTextPtr( ptrField, ZodiacSigns[nIndex].pszAttributes);
	FldRecalculateField( ptrField, true );
}

/**
  Draw one entry-line for one zodiac.
  Allows additionally to invert the entry, this is to mark the current one.
  \param nIndex    -> Index into the list of zodiacs
  \param fInverted -> Flag : shall the entry be inverted 
*/ 

void DrawZodiacListEntry( int nIndex , int fInverted)
{
	char szDateString[60];
	StrPrintF( szDateString,"%02d.%02d.-%02d.%02d.",
				ZodiacSigns[nIndex].Start.day,
				ZodiacSigns[nIndex].Start.month,
				ZodiacSigns[nIndex].End.day,
				ZodiacSigns[nIndex].End.month );
	if( fInverted )
	{
		WinDrawChars(szDateString,StrLen(szDateString),0, nIndex * 11 + 12 );
		WinDrawChars(ZodiacSigns[nIndex].szGerman,StrLen(ZodiacSigns[nIndex].szGerman),60, nIndex * 11 + 12 );
		WinDrawChars(ZodiacSigns[nIndex].szEnglish,StrLen(ZodiacSigns[nIndex].szEnglish),115, nIndex * 11 + 12 );

	}
	else
	{
		WinDrawInvertedChars(szDateString,StrLen(szDateString),0, nIndex * 11 + 12 );
		WinDrawInvertedChars(ZodiacSigns[nIndex].szGerman,StrLen(ZodiacSigns[nIndex].szGerman),60, nIndex * 11 + 12 );
		WinDrawInvertedChars(ZodiacSigns[nIndex].szEnglish,StrLen(ZodiacSigns[nIndex].szEnglish),115, nIndex * 11 + 12 );
	}
}


/**
  Generate an index into the array of zodiacs out of a given date. 
  \param ptr -> ptr to a given time 

*/ 

int DateTimeToIndex( DateTimePtr pPtr)
{
	int i;
	UInt32 DaysToday;
	UInt32 DaysStart;
	UInt32 DaysEnd;
	DateType Today;

	Today.day   = pPtr->day;
	Today.month = pPtr->month;
	Today.year  = 0;

	DaysToday = DateToDays( Today);

	for( i = 0; i < 12; ++i )
	{
		DaysStart = DateToDays( ZodiacSigns[i].Start );
		DaysEnd   = DateToDays( ZodiacSigns[i].End );
		if( ( DaysToday >= DaysStart ) && ( DaysToday <= DaysEnd ))
		{
			return i;
		}
		/* Special Handling for capricorn which has the year-break in it ... */
/*		if( i == 11 )  originally , but next if is better ... */
		if( DaysEnd < DaysStart )
		{
			if(( DaysToday >= DaysStart ) || ( DaysToday <= DaysEnd ))
			{
				return i;
			}

		}
	}
}

/**
  Draws the details of a zodiac.
*/ 

Boolean DetailFormHandler( EventPtr event )
{
    FormPtr formular;
    Boolean handled = false;
    static Boolean bNix    = true;
    ControlPtr ctlZeiger;
	DateTimePtr pDateTimeSelect;

    switch( event->eType )
    {
		case keyDownEvent:		// Handle all events concerning keypresses
		    formular = FrmGetActiveForm();
			switch( event->data.keyDown.chr )
			{
				case vchrPageDown:  // Keydown and down on the 5way-Navi
					ZodiacIndex++;
					ZodiacIndex %= 12;
					DrawZodiac( ZodiacIndex, formular );
					handled = true;
					break;
				case vchrPageUp:	// KeyUp and Up on the 5way-Navi
					if( ZodiacIndex )
						ZodiacIndex--;
					else
						ZodiacIndex = 11;
					DrawZodiac( ZodiacIndex, formular );
					handled = true;

					break;
				case vchrNavChange: // 5-way-Key : Select-Key pressed (to be precise:  released) ?
					if (NavSelectPressed(event))
					{
						FrmGotoForm( ListFormular );
						handled = true;
					}
					break;
			}
			break;
    	case frmOpenEvent:
        	formular = FrmGetActiveForm();
        	FrmDrawForm( formular );
			DrawZodiac( ZodiacIndex, formular );
        	handled = true;
        	break;
  		case ctlSelectEvent:
        	formular = FrmGetActiveForm();
        	switch( event->data.ctlSelect.controlID )
        	{
        		case ButtonPrev:
					if( ZodiacIndex > 0 )
					{
						ZodiacIndex--;
					}
					else
					{
						ZodiacIndex=11;
					}
					DrawZodiac( ZodiacIndex , formular);
	            	handled = true;
            		break;
        		case ButtonNext:
					ZodiacIndex++;
					if( ZodiacIndex > 11 )
					{
						ZodiacIndex=0;
					}
					DrawZodiac( ZodiacIndex, formular );
	            	handled = true;
            		break;
				case ButtonDate:
					pDateTimeSelect = MemPtrNew( 20 );
					TimSecondsToDateTime( TimGetSeconds(), pDateTimeSelect );
					if( SelectDay( selectDayByDay, &pDateTimeSelect->month,
												&pDateTimeSelect->day,
												&pDateTimeSelect->year,
												"Whle Tag fr Sternzeichen"))
					{

						ZodiacIndex = DateTimeToIndex( pDateTimeSelect);
						DrawZodiac( ZodiacIndex, formular );
					}
					MemPtrFree( pDateTimeSelect );
					handled = true;
					break;
				case ButtonList:
					FrmGotoForm( ListFormular );
					handled = true;
					break;
        	}
        	break;
    }
    return handled;
}


Boolean ListFormHandler( EventPtr event )
{
    FormPtr formular;
    Boolean handled = false;
    static Boolean bNix    = true;
    ControlPtr ctlZeiger;
	int i;
    Boolean fRedraw = false;

    switch( event->eType )
    {
    	case frmOpenEvent:
        	formular = FrmGetActiveForm();
        	FrmDrawForm( formular );
            fRedraw = true;
        	handled = true;
        	break;
		case keyDownEvent:
			switch( event->data.keyDown.chr )
			{
				case vchrPageDown:
					ZodiacIndex++;
					ZodiacIndex %= 12;
					fRedraw = true;
					handled = true;
					break;
				case vchrPageUp:
					if( ZodiacIndex )
						ZodiacIndex--;
					else
						ZodiacIndex = 11;
					fRedraw = true;
					handled = true;
					break;
				case vchrNavChange: // 5-way
					if (NavSelectPressed(event))
					{
						FrmGotoForm( DetailFormular );
	            		handled = true;
					}
					break;
			}
			break;
  		case ctlSelectEvent:
        	formular = FrmGetActiveForm();
			switch( event->data.ctlSelect.controlID )
        	{
        		case ButtonDetail:
					FrmGotoForm( DetailFormular );
	            	handled = true;
            		break;

        	}
        	break;
    }
    if( fRedraw )
    {
			for( i = 0; i < 12; ++i )
			{
				if( i == ZodiacIndex )
				{
					DrawZodiacListEntry( i, false );
				}
				else
				{
					DrawZodiacListEntry( i, true );
				}
			}
	}
    return handled;
}


Boolean ApplicationHandleEvent( EventPtr event )
{
    FormPtr formular;
    int formularId;
    Boolean handled = false;

    if ( event->eType == frmLoadEvent )
    {
        formularId = event->data.frmLoad.formID ;
        formular   = FrmInitForm( formularId );
        FrmSetActiveForm( formular );

        switch( formularId )
        {
        	case ListFormular:
            	FrmSetEventHandler( formular, ListFormHandler );
            	break;
			case DetailFormular:
            	FrmSetEventHandler( formular, DetailFormHandler );
            	break;
			default:
				break;
        }
        handled =true;
    }
    return handled;
}


UInt32 PilotMain( UInt16 cmd, void * cmdPBP, UInt16 launchFlags )
{
    EventType event;
    Err error;

    if( cmd == sysAppLaunchCmdNormalLaunch )
    {
        StartApplication();
        do
        {
            EvtGetEvent( &event, evtWaitForever );
            if( !SysHandleEvent( &event ))
            {
	            if(!ApplicationHandleEvent(&event))
                {
                	FrmDispatchEvent( &event );
                }
            }
        }
        while( event.eType != appStopEvent );
        StopApplication();
    }
    return 0;
}
